
state("soniccd")
{
    byte frames : "soniccd.exe", 0x804EFB;
    byte millisecs : "soniccd.exe", 0xF9F2F0;
    byte seconds : "soniccd.exe", 0xE2A378;
    byte mins : "soniccd.exe", 0xF9F2F3;
	int timeBonus : "soniccd.exe", 0xC3DEFC;
    int ringBonus : "soniccd.exe", 0xC3DF00;
    byte levelID : "soniccd.exe", 0xF9F294;
    byte bhpGood : "soniccd.exe", 0xC40EB8;
    byte bhpBad : "soniccd.exe", 0xC40EFC;
    int mode : "soniccd.exe", 0xC396E0;
}

startup
{
    refreshRate = 60;
    string logfile = Directory.GetCurrentDirectory() + "\\SonicCDAutoSplitter.log";
    if ( File.Exists( logfile ) ) {
        File.Delete( logfile );
    }
    print(logfile);
    settings.Add("accurate", true, "Use accurate timing (note IGT won't match splits)", null);


    Action<string> DebugOutput = (text) => {
        print("[Sonic CD Splitter] "+text);
        string time = System.DateTime.Now.ToString("dd/mm/yy hh:mm:ss:fff");
        File.AppendAllText(logfile, "[" + time + "]: " + text + "\r\n");
        
    };


    vars.DebugOutput = DebugOutput;
}

init
{
    vars.DebugOutput("init detected");
    vars.totalms = 0;
    vars.boss = 0;
    vars.running = false;
}
update
{
    var start = false;
    var split = false;
    var reset = false;
    var addition = 0;
    string quote = "\"";  

    // Stores the curent phase the timer is in, so we can use the old one on the next frame.
	current.timerPhase = timer.CurrentPhase;
    current.msfromframes = Math.Floor(current.frames * (10.0/6.0));
    // Reset some variables when the timer is started
    
	if ((old.timerPhase != current.timerPhase && old.timerPhase != TimerPhase.Paused) && current.timerPhase == TimerPhase.Running) {

        vars.DebugOutput("run start detected");
        vars.totalms = 0;
        vars.splitms = 0;
        vars.carriedcs = 0;
        vars.boss = 0; 
        vars.running = true;
        current.startingms = current.msfromframes;
        vars.DebugOutput(String.Format("starting cs {0}", current.msfromframes));
    }
    if ( current.timerPhase != TimerPhase.Running && old.timerPhase == TimerPhase.Running ) {
        vars.DebugOutput("run stop detected");
        vars.running = false;
    }
    if ( !vars.running ) {
        return false;
    }
    
    if ( settings["accurate"] || (current.levelID == 68 || current.levelID == 69) ) {
        if ( current.msfromframes > old.msfromframes ) {
            addition = (int) ( ( current.msfromframes - old.msfromframes ) * 10);
        } else if ( current.msfromframes < old.msfromframes ) {
            addition = (int) ( ( (current.msfromframes + 100) - old.msfromframes ) * 10);
        }
    } else {
        if ( current.msfromframes < old.msfromframes ) {
            addition = 1000;
        }
    }
    vars.totalms += addition;



    if (current.timeBonus == 0 && old.timeBonus > 0) {
        vars.DebugOutput(String.Format("Split IGT: {0:D2}\'{1:D2}\"{2:D2} Logged: {3}",current.mins, current.seconds, current.millisecs, TimeSpan.FromMilliseconds(vars.totalms - vars.splitms).ToString("mm'\\\''ss'\\\"'ff")));
        vars.DebugOutput(String.Format("cs carried: {0}", current.msfromframes));
        vars.splitms = vars.totalms;
        vars.carriedcs = current.msfromframes;
        split = true;
    }
    
    if (current.levelID == 68 || current.levelID == 69)
    {
        if (vars.boss == 0) {
            if (current.bhpGood == 4 && old.bhpGood == 0)
            {
                if (settings["debug"]) {
                    vars.DebugOutput("good future boss detected");
                }
                vars.boss = 1;
            }
            if (current.bhpBad == 4 && old.bhpBad == 0)
            {
                if (settings["debug"]) {
                    vars.DebugOutput("bad future boss detected");
                }
                vars.boss = 2;
            }
        }
            
        if (vars.boss == 1) { //good future
            if (current.bhpGood == 0 && old.bhpGood == 1 && !(current.mins == 0 && current.seconds == 0)) { 
                if (settings["debug"]) {
                    vars.DebugOutput("final split - boss death detected");
                }
                split = true; 
            }
        }

            
        if (vars.boss == 2) { //bad future
            if (current.bhpBad == 0 && old.bhpBad == 1 && !(current.mins == 0 && current.seconds == 0)) { 
                if (settings["debug"]) {
                    vars.DebugOutput("final split - boss death detected");
                }
                split = true;
            }
        }
        if ( split && !settings["accurate"] ) {
            vars.totalms += vars.carriedcs * 10;
        }
    }

    current.start = start;
    current.reset = reset;
    current.split = split;
	return true;
}

gameTime
{
    //our saved tally of the overall run + any extra time from deaths this level + the current game time
    return TimeSpan.FromMilliseconds(vars.totalms);
}

isLoading
{
    //stop the timer counting in real time, we're updating the clock manually
	return true;
}

split
{

    if ( current.split ) {
        current.split = false;
        return true;
    }

    
}
